"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const format_1 = require("../format");
const reporter_1 = require("../reporter");
const host_1 = require("./host");
class LanguageService {
    constructor(config) {
        this.host = new host_1.LanguageServiceHost(config);
        this.service = ts.createLanguageService(this.host, ts.createDocumentRegistry());
    }
    check(path, reportErrors) {
        const { service } = this;
        this.host.invalidate(path);
        const diagnostics = [
            ...service.getSemanticDiagnostics(path),
            ...service.getSyntacticDiagnostics(path)
        ];
        const formatted = format_1.formatDiagnostics(diagnostics, process.cwd());
        if (reportErrors && diagnostics.length > 0) {
            reporter_1.reportDiagnostics(diagnostics);
        }
        return {
            diagnostics: diagnostics.length > 0
                ? formatted
                : null
        };
    }
}
exports.LanguageService = LanguageService;
//# sourceMappingURL=index.js.map