"use strict";
const tslib_1 = require("tslib");
const tsc_1 = require("../compiler/tsc");
const config_loader_1 = require("../config-loader");
const index_1 = require("../service/index");
const compilers = new Map();
const services = new Map();
function getCompiler(tsConfig) {
    let compiler = compilers.get(tsConfig);
    if (!compiler) {
        compiler = config_loader_1.loadConfiguration(tsConfig).then(config => new tsc_1.TypeScriptCompiler(config));
        compilers.set(tsConfig, compiler);
    }
    return compiler;
}
function getService(tsConfig) {
    let service = services.get(tsConfig);
    if (!service) {
        service = config_loader_1.loadConfiguration(tsConfig).then(config => new index_1.LanguageService(config));
        services.set(tsConfig, service);
    }
    return service;
}
const handler = {
    compile({ file, reportErrors, tsConfig }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const compiler = yield getCompiler(tsConfig);
            return compiler.compile(file, reportErrors);
        });
    },
    typeCheck({ file, reportErrors, tsConfig }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const service = yield getService(tsConfig);
            return service.check(file, reportErrors);
        });
    }
};
module.exports = handler;
//# sourceMappingURL=launcher.js.map